import type { TRPCClientErrorLike, CreateTRPCProxyClient } from '@trpc/client';
import type { AnyRouter } from '@trpc/server';
import { createQuery, createMutation, createInfiniteQuery, createQueries, CreateQueryOptions, CreateMutationOptions, CreateInfiniteQueryOptions, InvalidateQueryFilters, FetchQueryOptions, FetchInfiniteQueryOptions, InfiniteData, RefetchQueryFilters, RefetchOptions, ResetOptions, CancelOptions, Updater, SetDataOptions, QueryClient } from '@tanstack/svelte-query';
type WithNevers<T, V> = {
    [K in keyof T]: Exclude<T[K], undefined> extends V ? never : T[K] extends Record<string, unknown> ? Without<T[K], V> : T[K];
};
type Without<T, V, I = WithNevers<T, V>> = Pick<I, {
    [K in keyof I]: I[K] extends never ? never : K;
}[keyof I]>;
type HasQuery = {
    query: (...args: any[]) => any;
};
type HasMutate = {
    mutate: (...args: any[]) => any;
};
type HasSubscribe = {
    subscribe: (...args: any[]) => any;
};
type OnlyQueries<TClient> = Without<TClient, HasMutate | HasSubscribe>;
type QueryType = 'query' | 'infinite' | 'any';
type QueryKey = [
    string[],
    {
        input?: unknown;
        type?: Exclude<QueryType, 'any'>;
    }?
];
type GetQueryKey<TInput = undefined> = {
    getQueryKey: TInput extends undefined ? () => QueryKey : (input: TInput, type?: QueryType) => QueryKey;
};
declare const ContextProcedureNames: {
    readonly client: "client";
    readonly fetch: "fetch";
    readonly prefetch: "prefetch";
    readonly fetchInfinite: "fetchInfinite";
    readonly prefetchInfinite: "prefetchInfinite";
    readonly invalidate: "invalidate";
    readonly refetch: "refetch";
    readonly reset: "reset";
    readonly cancel: "cancel";
    readonly setData: "setData";
    readonly getData: "getData";
    readonly setInfiniteData: "setInfiniteData";
    readonly getInfiniteData: "getInfiniteData";
};
type ContextProcedures<TInput = undefined, TOutput = undefined, TError = undefined> = {
    [ContextProcedureNames.fetch](input: TInput, opts?: FetchQueryOptions<TInput, TError, TOutput>): Promise<TOutput>;
    [ContextProcedureNames.prefetch](input: TInput, opts?: FetchQueryOptions<TInput, TError, TOutput>): Promise<void>;
    [ContextProcedureNames.fetchInfinite](input: TInput, opts?: FetchInfiniteQueryOptions<TInput, TError, TOutput>): Promise<InfiniteData<TOutput>>;
    [ContextProcedureNames.prefetchInfinite](input: TInput, opts?: FetchInfiniteQueryOptions<TInput, TError, TOutput>): Promise<void>;
    [ContextProcedureNames.invalidate](input?: TInput, filters?: InvalidateQueryFilters): Promise<void>;
    [ContextProcedureNames.refetch](input?: TInput, filters?: RefetchQueryFilters, opts?: RefetchOptions): Promise<void>;
    [ContextProcedureNames.reset](input?: TInput, opts?: ResetOptions): Promise<void>;
    [ContextProcedureNames.cancel](input?: TInput, opts?: CancelOptions): Promise<void>;
    [ContextProcedureNames.setData](input: TInput, updater: Updater<TOutput | undefined, TOutput | undefined>, opts?: SetDataOptions): void;
    [ContextProcedureNames.getData](input?: TInput): TOutput | undefined;
    [ContextProcedureNames.setInfiniteData](input: TInput, updater: Updater<InfiniteData<TOutput> | undefined, InfiniteData<TOutput> | undefined>, opts?: SetDataOptions): void;
    [ContextProcedureNames.getInfiniteData](input?: TInput): InfiniteData<TOutput> | undefined;
};
type AddContextPropTypes<TClient, TError> = {
    [K in keyof TClient]: TClient[K] extends HasQuery ? ContextProcedures<Parameters<TClient[K]['query']>[0], Awaited<ReturnType<TClient[K]['query']>>, TError> : AddContextPropTypes<TClient[K], TError> & Pick<ContextProcedures, typeof ContextProcedureNames.invalidate>;
};
type UseContext<TClient, TError> = AddContextPropTypes<OnlyQueries<TClient>, TError> & Pick<ContextProcedures, typeof ContextProcedureNames.invalidate> & {
    [ContextProcedureNames.client]: TClient;
};
type CreateQueryOptionsForUseQueries<TInput, TError> = Omit<CreateQueryOptions<TInput, TError>, 'context'>;
type UseQueriesRecord<TClient, TError> = {
    [K in keyof TClient]: TClient[K] extends HasQuery ? (input: Parameters<TClient[K]['query']>[0], opts?: CreateQueryOptionsForUseQueries<Awaited<ReturnType<TClient[K]['query']>>, TError>) => CreateQueryOptionsForUseQueries<Awaited<ReturnType<TClient[K]['query']>>, TError> : UseQueriesRecord<TClient[K], TError>;
};
type UseQueries<TClient, TError> = <TOpts extends CreateQueryOptionsForUseQueries<any, any>[]>(queriesCallback: (t: UseQueriesRecord<OnlyQueries<TClient>, TError>) => readonly [...TOpts], context?: CreateQueryOptions['context']) => ReturnType<typeof createQueries<TOpts>>;
declare const ProcedureNames: {
    readonly query: "useQuery";
    readonly infiniteQuery: "useInfiniteQuery";
    readonly mutate: "useMutation";
    readonly subscribe: "useSubscription";
    readonly queryKey: "getQueryKey";
    readonly context: "useContext";
    readonly queries: "useQueries";
};
type UseQueryProcedure<TInput, TOutput, TError> = {
    [ProcedureNames.query]: (input: TInput, opts?: CreateQueryOptions<TOutput, TError>) => ReturnType<typeof createQuery<TOutput, TError>>;
};
type UseInfiniteQueryProcedure<TInput, TOutput, TError> = TInput extends {
    cursor?: any;
} ? {
    [ProcedureNames.infiniteQuery]: (input: Omit<TInput, 'cursor'>, opts?: CreateInfiniteQueryOptions<TOutput, TError>) => ReturnType<typeof createInfiniteQuery<TOutput, TError>>;
} : {};
type QueryProcedures<TInput, TOutput, TError> = UseQueryProcedure<TInput, TOutput, TError> & UseInfiniteQueryProcedure<TInput, TOutput, TError> & GetQueryKey<TInput>;
type UseMutationProcedure<TInput, TOutput, TError, TContext = unknown> = {
    [ProcedureNames.mutate]: (opts?: CreateMutationOptions<TOutput, TError, TInput, TContext>) => ReturnType<typeof createMutation<TOutput, TError, TInput, TContext>>;
};
type UseTRPCSubscriptionOptions<TOutput, TError> = {
    enabled?: boolean;
    onStarted?: () => void;
    onData: (data: TOutput) => void;
    onError?: (err: TError) => void;
};
type GetSubscriptionOutput<TOpts> = TOpts extends unknown & Partial<infer A> ? A extends {
    onData: any;
} ? Parameters<A['onData']>[0] : never : never;
type UseSubscriptionProcedure<TInput, TOutput, TError> = {
    [ProcedureNames.subscribe]: (input: TInput, opts?: UseTRPCSubscriptionOptions<TOutput, TError>) => void;
};
type AddQueryPropTypes<TClient, TError> = TClient extends Record<any, any> ? {
    [K in keyof TClient]: TClient[K] extends HasQuery ? QueryProcedures<Parameters<TClient[K]['query']>[0], Awaited<ReturnType<TClient[K]['query']>>, TError> : TClient[K] extends HasMutate ? UseMutationProcedure<Parameters<TClient[K]['mutate']>[0], Awaited<ReturnType<TClient[K]['mutate']>>, TError> : TClient[K] extends HasSubscribe ? UseSubscriptionProcedure<Parameters<TClient[K]['subscribe']>[0], GetSubscriptionOutput<Parameters<TClient[K]['subscribe']>[1]>, TError> : AddQueryPropTypes<TClient[K], TError> & GetQueryKey;
} : TClient;
export declare function svelteQueryWrapper<TRouter extends AnyRouter>(client: CreateTRPCProxyClient<TRouter>, queryClient?: QueryClient): AddQueryPropTypes<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>> & (AddQueryPropTypes<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>> extends Record<any, any> ? {
    useContext(): UseContext<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
    useQueries: UseQueries<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
} : {});
export {};
